/*
 * Ensys srl
 * 
 * Questo esempio recupera il dato della temperatura dal sensore installato nella IndRsp
 * 
 * Verificare di aver abilitato la porta I2C0, ad esempio integrando nel file
 * 	/boot/config.txt 
 * le seguenti righe:
 * 	#i2c0 - periferiche
 * 	dtparam=i2c0=on
 * 	dtoverlay=i2c0-bcm2708,pins_28_29
 * 
 * Per compilare questo file utilizzare il comando:
 * 	gcc -o test mcp9808t.c
 * Poi eseguire:
 * 	./test
 */
 

#include <unistd.h>
#include <stdio.h>
#include <stdlib.h>
#include <stdint.h>
#include <fcntl.h>
#include <sys/ioctl.h>
#include <linux/i2c-dev.h>

#define HD__DEV__ADDR       (0x1Fu)
#define HD__ID__ADDR        (0x06u)
#define HD__ID__VAL         (0x0054u)

int32_t i2c__byte_read_byte(int32_t fd, uint8_t add, uint8_t *pdata){
        int32_t ritorno = -1;
        if(write(fd,&add,1u)==1u){
                if(read(fd,pdata,1u)==1u){
                        ritorno = 0;
                }
        }
        return ritorno;
}
int32_t i2c__byte_read_word(int32_t fd, uint8_t add, uint16_t *pdata){
        uint8_t buffer[2u];
        int32_t ritorno = -1;
        if(write(fd,&add,1u)==1u){
                if(read(fd,buffer,2u)==2u){
                        *pdata = (buffer[0]<<8) | (buffer[1]);
                        ritorno = 0;
                }
        }
        return ritorno;
}
int32_t i2c__byte_read_buffer(int32_t fd, uint8_t add, uint8_t buffer[], uint8_t dim_buffer){
        int32_t ritorno = -1;
        if(write(fd,&add,1u)==1u){
                if(read(fd,buffer,dim_buffer)==dim_buffer){
                        ritorno = 0;
                }
        }
        return ritorno;
}
int32_t i2c__byte_write_byte(int32_t fd, uint8_t add, uint8_t data){
        uint8_t buffer[2];
        int32_t ritorno = -1;
        buffer[0]=add;
        buffer[1]=data;
        if(write(fd,buffer,2u)==2u){
                ritorno = 0;
        }
        return ritorno;
}

int32_t main(int32_t argc, uint8_t *argv[])
{
	int32_t fd;
	uint8_t  buffer[10];
        int16_t data;
        float temp;
        
        /* Apertura device */
       	if((fd=open("/dev/i2c-0", O_RDWR))<0)           printf("Error open\n");
        if(ioctl(fd, I2C_SLAVE, HD__DEV__ADDR)<0)        printf("Error set addr\n");
        
        /* Lettura ID */
        /* Lettura ID */
        if(i2c__byte_read_word(fd, HD__ID__ADDR, &data)==0){
                if(data==HD__ID__VAL)      printf("OK, trovato MCP9808T!\n");
        }


        /* Set resolution */
        if(i2c__byte_write_byte(fd, 0x08, 3u)==0){
                printf(" 0x%02x\n", data);
        }

         for(int32_t i=0;i<10000;){
                /* Lettura temperatura */
                if(i2c__byte_read_word(fd, 0x05, &data)!=0){
                        printf("Errore\n");
                        break;
                }
                else{
                        /* Trasformazione in °C */
                        if(data!=0xFFFF){
                                temp = data & 0x0FFF;
                                temp /= 16.0;
                                if (data & 0x1000)
                                        temp = 256 - temp;
                        }

                        printf(" %04d %f \n", data, temp);
                        i++;
                }               
        }

       
        /* Chiusura device */
        close(fd);

	return (0);
}
